-- module Encoding.Base45
--   ( enc45
--   , dec45
--   ) where

-- import Data.Char (chr, ord)
-- import Data.List (elemIndex)
-- import Data.Maybe (fromJust)

-- base45Alphabet :: String
-- base45Alphabet = ['0'..'9'] ++ ['A'..'Z'] ++ " " ++ "$%*+-./:"

-- enc45 :: String -> String
-- enc45 = concatMap (reverse . encodeChunk) . chunkBy 2 . map ord

-- chunkBy :: Int -> [a] -> [[a]]
-- chunkBy _ [] = []
-- chunkBy n xs = take n xs : chunkBy n (drop n xs)

-- encodeChunk :: [Int] -> String
-- encodeChunk [x1, x2] = map (base45Alphabet !!) [c, b, a]
--   where
--     n = x1 * 256 + x2
--     a = n `mod` 45
--     b = (n `div` 45) `mod` 45
--     c = n `div` 2025
-- encodeChunk [x1] = map (base45Alphabet !!) [b, a]
--   where
--     a = x1 `mod` 45
--     b = x1 `div` 45
-- encodeChunk _ = error "Invalid chunk length"

-- -- Decode
-- dec45 :: String -> String
-- dec45 encoded = bytesToChars decodedBytes
--   where
--     numericValues = map charToNumeric encoded
--     groups = groupIntoThrees numericValues
--     base45Values = map toBase45 groups
--     decodedBytes = concatMap fromBase45 base45Values

-- charToNumeric :: Char -> Int
-- charToNumeric c = case c of
--   '0' -> 0
--   '1' -> 1
--   '2' -> 2
--   '3' -> 3
--   '4' -> 4
--   '5' -> 5
--   '6' -> 6
--   '7' -> 7
--   '8' -> 8
--   '9' -> 9
--   'A' -> 10
--   'B' -> 11
--   'C' -> 12
--   'D' -> 13
--   'E' -> 14
--   'F' -> 15
--   'G' -> 16
--   'H' -> 17
--   'I' -> 18
--   'J' -> 19
--   'K' -> 20
--   'L' -> 21
--   'M' -> 22
--   'N' -> 23
--   'O' -> 24
--   'P' -> 25
--   'Q' -> 26
--   'R' -> 27
--   'S' -> 28
--   'T' -> 29
--   'U' -> 30
--   'V' -> 31
--   'W' -> 32
--   'X' -> 33
--   'Y' -> 34
--   'Z' -> 35
--   ' ' -> 36
--   '$' -> 37
--   '%' -> 38
--   '*' -> 39
--   '+' -> 40
--   '-' -> 41
--   '.' -> 42
--   '/' -> 43
--   ':' -> 44
--   _ -> error "Invalid Base45 character"

-- groupIntoThrees :: [Int] -> [[Int]]
-- groupIntoThrees [] = []
-- groupIntoThrees xs
--   | length xs < 3 = [xs ++ replicate (3 - length xs) 0]
--   | otherwise = take 3 xs : groupIntoThrees (drop 3 xs)

-- toBase45 :: [Int] -> Int
-- toBase45 [c, d, e] = c + d * 45 + e * (45 * 45)

-- fromBase45 :: Int -> [Int]
-- fromBase45 n
--   | n >= 256 = fromBase45 (n `div` 256) ++ [n `mod` 256]
--   | otherwise = [n]

-- bytesToChars :: [Int] -> String
-- bytesToChars = map chr

module Encoding.Base45
  ( enc45
  , dec45
  ) where

import Data.Char (chr, ord)
import Data.ByteString (ByteString)
import qualified Data.ByteString as B
import qualified Data.ByteString.Char8 as BC
import Data.List (unfoldr)
import Data.Maybe (fromJust)
import Data.Word (Word8)

base45Alphabet :: ByteString
base45Alphabet = BC.pack $ ['0'..'9'] ++ ['A'..'Z'] ++ " " ++ "$%*+-./:"

enc45 :: ByteString -> ByteString
enc45 = BC.concat . map (BC.reverse . encodeChunk . map fromIntegral) . chunkBy 2 . B.unpack

chunkBy :: Int -> [a] -> [[a]]
chunkBy _ [] = []
chunkBy n xs = take n xs : chunkBy n (drop n xs)

encodeChunk :: [Int] -> ByteString
encodeChunk [x1, x2] = BC.pack $ map (BC.index base45Alphabet) [c, b, a]
  where
    n = x1 * 256 + x2
    a = n `mod` 45
    b = (n `div` 45) `mod` 45
    c = n `div` 2025
encodeChunk [x1] = BC.pack $ map (BC.index base45Alphabet) [b, a]
  where
    a = x1 `mod` 45
    b = x1 `div` 45
encodeChunk _ = error "Invalid chunk length"

dec45 :: ByteString -> ByteString
dec45 encoded = B.pack $ map fromIntegral decodedBytes
  where
    numericValues = map charToNumeric (BC.unpack encoded)
    groups = groupIntoThrees numericValues
    base45Values = map toBase45 groups
    decodedBytes = concatMap fromBase45 base45Values

charToNumeric :: Char -> Int
charToNumeric c = fromJust $ BC.elemIndex c base45Alphabet

groupIntoThrees :: [Int] -> [[Int]]
groupIntoThrees [] = []
groupIntoThrees xs
  | length xs < 3 = [xs ++ replicate (3 - length xs) 0]
  | otherwise = take 3 xs : groupIntoThrees (drop 3 xs)

toBase45 :: [Int] -> Int
toBase45 [c, d, e] = c + d * 45 + e * (45 * 45)
toBase45 _ = error "Invalid Base45 group"

fromBase45 :: Int -> [Int]
fromBase45 n = unfoldr (\x -> if x == 0 then Nothing else Just (x `mod` 256, x `div` 256)) n

bytesToChars :: [Int] -> [Char]
bytesToChars = map chr

